@@ -3,7 +3,7 @@
 import json
 import yaml
 from typing import Optional, Union, Dict, Tuple, Any
-from argparse import Action, Namespace
+from argparse import Action
 
 from .actions import _is_action_value_list
 from .jsonschema import ActionJsonSchema
@@ -14,15 +14,10 @@ from .optionals import (
     jsonschemaValidationError,
 )
 from .util import (
-    dict_to_namespace,
-    namespace_to_dict,
     yamlParserError,
     yamlScannerError,
     ParserError,
     Path,
-    _get_key_value,
-    _flat_namespace_to_dict,
-    _dict_to_flat_namespace,
 )
 
 
@@ -76,7 +71,7 @@ class ActionJsonnet(Action):
                     try:
                         schema = yaml.safe_load(schema)
                     except (yamlParserError, yamlScannerError) as ex:
-                        raise ValueError('Problems parsing schema :: '+str(ex)) from ex
+                        raise ValueError(f'Problems parsing schema :: {ex}') from ex
                 jsonvalidator.check_schema(schema)
                 self._validator = ActionJsonSchema._extend_jsonvalidator_with_default(jsonvalidator)(schema)
             else:
@@ -107,10 +102,8 @@ class ActionJsonnet(Action):
         ext_vars = {}
         if self._ext_vars is not None:
             try:
-                if isinstance(cfg, dict):
-                    cfg = _flat_namespace_to_dict(_dict_to_flat_namespace(cfg))
-                ext_vars = _get_key_value(cfg, self._ext_vars)
-            except (KeyError, AttributeError):
+                ext_vars = cfg[self._ext_vars]
+            except KeyError:
                 pass
         if not islist:
             value = [value]
@@ -123,14 +116,12 @@ class ActionJsonnet(Action):
                 value[num] = val
             except (TypeError, RuntimeError, yamlParserError, yamlScannerError, jsonschemaValidationError) as ex:
                 elem = '' if not islist else ' element '+str(num+1)
-                raise TypeError('Parser key "'+self.dest+'"'+elem+': '+str(ex)) from ex
+                raise TypeError(f'Parser key "{self.dest}"{elem}: {ex}') from ex
         return value if islist else value[0]
 
 
     @staticmethod
-    def split_ext_vars(
-        ext_vars: Optional[Union[Dict[str, Any], Namespace]],
-    ) -> Tuple[Dict[str, Any], Dict[str, Any]]:
+    def split_ext_vars(ext_vars: Optional[Dict[str, Any]]) -> Tuple[Dict[str, Any], Dict[str, Any]]:
         """Splits an ext_vars dict into the ext_codes and ext_vars required by jsonnet.
 
         Args:
@@ -138,8 +129,6 @@ class ActionJsonnet(Action):
         """
         if ext_vars is None:
             ext_vars = {}
-        elif isinstance(ext_vars, Namespace):
-            ext_vars = namespace_to_dict(ext_vars)
         ext_codes = {k: json.dumps(v) for k, v in ext_vars.items() if not isinstance(v, str)}
         ext_vars = {k: v for k, v in ext_vars.items() if isinstance(v, str)}
         return ext_vars, ext_codes
@@ -148,9 +137,9 @@ class ActionJsonnet(Action):
     def parse(
         self,
         jsonnet: Union[str, Path],
-        ext_vars: Union[Dict[str, Any], Namespace] = None,
+        ext_vars: Optional[Dict[str, Any]] = None,
         with_meta: bool = False,
-    ) -> Namespace:
+    ) -> Dict:
         """Method that can be used to parse jsonnet independent from an ArgumentParser.
 
         Args:
@@ -179,11 +168,11 @@ class ActionJsonnet(Action):
         try:
             values = yaml.safe_load(_jsonnet.evaluate_snippet(fname, snippet, ext_vars=ext_vars, ext_codes=ext_codes))
         except RuntimeError as ex:
-            raise ParserError('Problems evaluating jsonnet "'+fname+'" :: '+str(ex)) from ex
+            raise ParserError(f'Problems evaluating jsonnet "{fname}" :: {ex}') from ex
         if self._validator is not None:
             self._validator.validate(values)
         if with_meta:
-            if isinstance(values, dict) and fpath is not None:
+            if fpath is not None:
                 values['__path__'] = fpath
             values['__orig__'] = snippet
-        return dict_to_namespace(values)
+        return values
