@@ -1,5 +1,4 @@
 #!/usr/bin/env python3
-# pylint: disable=unsubscriptable-object
 
 import json
 import pathlib
@@ -9,7 +8,7 @@ import yaml
 from calendar import Calendar
 from datetime import datetime
 from enum import Enum
-from typing import Any, Callable, Dict, Iterable, List, Optional, Sequence, Tuple, Type, Union
+from typing import Any, Callable, Dict, Iterable, List, Mapping, Optional, Sequence, Tuple, Type, Union
 from jsonargparse_tests.base import *
 from jsonargparse.typehints import is_optional, Literal
 
@@ -102,7 +101,7 @@ class TypeHintsTests(unittest.TestCase):
 
 
     def test_dict(self):
-        parser = ArgumentParser(error_handler=None, parse_as_dict=True)
+        parser = ArgumentParser(error_handler=None)
         parser.add_argument('--dict', type=dict)
         self.assertEqual({}, parser.parse_args(['--dict={}'])['dict'])
         self.assertEqual({'a': 1, 'b': '2'}, parser.parse_args(['--dict={"a":1, "b":"2"}'])['dict'])
@@ -113,7 +112,7 @@ class TypeHintsTests(unittest.TestCase):
         class MyEnum(Enum):
             ab = 1
 
-        parser = ArgumentParser(error_handler=None, parse_as_dict=True)
+        parser = ArgumentParser(error_handler=None)
         parser.add_argument('--dict1', type=Dict[int, Optional[Union[float, MyEnum]]])
         parser.add_argument('--dict2', type=Dict[str, Union[bool, Path_fc]])
         cfg = parser.parse_args(['--dict1={"2":4.5, "6":"ab"}', '--dict2={"a":true, "b":"f"}'])
@@ -179,7 +178,7 @@ class TypeHintsTests(unittest.TestCase):
 
 
     def test_type_Any(self):
-        parser = ArgumentParser(error_handler=None, parse_as_dict=True)
+        parser = ArgumentParser(error_handler=None)
         parser.add_argument('--any', type=Any)
         self.assertEqual('abc', parser.parse_args(['--any=abc'])['any'])
         self.assertEqual(123, parser.parse_args(['--any=123'])['any'])
@@ -267,15 +266,16 @@ class TypeHintsTests(unittest.TestCase):
 
 
     def test_class_type(self):
-        parser = ArgumentParser(error_handler=None, parse_as_dict=True)
+        parser = ArgumentParser(error_handler=None)
         parser.add_argument('--op', type=Optional[List[Calendar]])
 
         class_path = '"class_path": "calendar.Calendar"'
+        expected = [{'class_path': 'calendar.Calendar', 'init_args': {'firstweekday': 0}}]
         cfg = parser.parse_args(['--op=[{'+class_path+'}]'])
-        self.assertEqual(cfg['op'], [{'class_path': 'calendar.Calendar', "init_args": {"firstweekday": 0}}])
+        self.assertEqual(cfg.as_dict()['op'], expected)
         cfg = parser.parse_args(['--op=["calendar.Calendar"]'])
-        self.assertEqual(cfg['op'], [{'class_path': 'calendar.Calendar', "init_args": {"firstweekday": 0}}])
-        cfg = parser.instantiate_subclasses(cfg)
+        self.assertEqual(cfg.as_dict()['op'], expected)
+        cfg = parser.instantiate_classes(cfg)
         self.assertIsInstance(cfg['op'][0], Calendar)
 
         with self.assertRaises(ParserError):
@@ -293,26 +293,26 @@ class TypeHintsTests(unittest.TestCase):
 
         init_args = '"init_args": {"firstweekday": 3}'
         cfg = parser.parse_args(['--op=[{'+class_path+', '+init_args+'}]'])
-        self.assertEqual(cfg['op'][0]['init_args'], {'firstweekday': 3})
-        cfg = parser.instantiate_subclasses(cfg)
+        self.assertEqual(cfg['op'][0]['init_args'].as_dict(), {'firstweekday': 3})
+        cfg = parser.instantiate_classes(cfg)
         self.assertIsInstance(cfg['op'][0], Calendar)
         self.assertEqual(3, cfg['op'][0].firstweekday)
 
-        parser = ArgumentParser(parse_as_dict=True)
+        parser = ArgumentParser(error_handler=None)
         parser.add_argument('--n.op', type=Optional[Calendar])
         cfg = parser.parse_args(['--n.op={'+class_path+', '+init_args+'}'])
-        cfg = parser.instantiate_subclasses(cfg)
+        cfg = parser.instantiate_classes(cfg)
         self.assertIsInstance(cfg['n']['op'], Calendar)
         self.assertEqual(3, cfg['n']['op'].firstweekday)
 
         parser = ArgumentParser()
         parser.add_argument('--op', type=Calendar)
         cfg = parser.parse_args(['--op={'+class_path+', '+init_args+'}'])
-        cfg = parser.instantiate_subclasses(cfg)
+        cfg = parser.instantiate_classes(cfg)
         self.assertIsInstance(cfg['op'], Calendar)
         self.assertEqual(3, cfg['op'].firstweekday)
 
-        cfg = parser.instantiate_subclasses(parser.parse_args([]))
+        cfg = parser.instantiate_classes(parser.parse_args([]))
         self.assertIsNone(cfg['op'])
 
 
@@ -431,17 +431,26 @@ class TypeHintsTmpdirTests(TempDirTestCase):
         with open('cal.yaml', 'w') as f:
             json.dump(cal, f)
 
-        parser = ArgumentParser(parse_as_dict=True, error_handler=None)
+        parser = ArgumentParser(error_handler=None)
         parser.add_argument('--data', type=Dict[str, Any], enable_path=True)
         parser.add_argument('--cal', type=Calendar, enable_path=True)
         cfg = parser.parse_args(['--data=data.yaml'])
         self.assertEqual('data.yaml', str(cfg['data'].pop('__path__')))
         self.assertEqual(data, cfg['data'])
-        cfg = parser.instantiate_subclasses(parser.parse_args(['--cal=cal.yaml']))
+        cfg = parser.instantiate_classes(parser.parse_args(['--cal=cal.yaml']))
         self.assertIsInstance(cfg['cal'], Calendar)
         self.assertRaises(ParserError, lambda: parser.parse_args(['--data=does-not-exist.yaml']))
 
 
+    def test_default_path_unregistered_type(self):
+        parser = ArgumentParser()
+        parser.add_argument('--path',
+                            type=path_type('drw', skip_check=True),
+                            default=Path('test', mode='drw', skip_check=True))
+        cfg = parser.parse_args([])
+        self.assertEqual('path: test\n', parser.dump(cfg))
+
+
     def test_class_type_with_default_config_files(self):
         config = {
             'class_path': 'calendar.Calendar',
@@ -459,12 +468,190 @@ class TypeHintsTmpdirTests(TempDirTestCase):
         parser.add_argument('--op', default='from default')
         parser.add_class_arguments(MyClass, 'data')
 
-        cfg = namespace_to_dict(parser.get_defaults())
+        cfg = parser.get_defaults()
         self.assertEqual(config_path, str(cfg['__default_config__']))
-        self.assertEqual(cfg['data']['cal'], config)
-        cfg = parser.dump(cfg)
-        self.assertIn('class_path: calendar.Calendar\n', cfg)
-        self.assertIn('firstweekday: 3\n', cfg)
+        self.assertEqual(cfg.data.cal.as_dict(), config)
+        dump = parser.dump(cfg)
+        self.assertIn('class_path: calendar.Calendar\n', dump)
+        self.assertIn('firstweekday: 3\n', dump)
+
+
+    def test_class_path_override_with_default_config_files(self):
+
+        class MyCalendar(Calendar):
+            def __init__(self, *args, param: str = '0', **kwargs):
+                super().__init__(*args, **kwargs)
+
+        import jsonargparse_tests
+        setattr(jsonargparse_tests, 'MyCalendar', MyCalendar)
+
+        config = {
+            'class_path': 'jsonargparse_tests.MyCalendar',
+            'init_args': {'firstweekday': 2, 'param': '1'},
+        }
+        config_path = os.path.join(self.tmpdir, 'config.yaml')
+        with open(config_path, 'w') as f:
+            json.dump({'cal': config}, f)
+
+        parser = ArgumentParser(error_handler=None, default_config_files=[config_path])
+        parser.add_argument('--cal', type=Optional[Calendar])
+
+        cfg = parser.instantiate_classes(parser.get_defaults())
+        self.assertIsInstance(cfg['cal'], MyCalendar)
+
+        cfg = parser.parse_args(['--cal={"class_path": "calendar.Calendar", "init_args": {"firstweekday": 3}}'])
+        self.assertEqual(type(parser.instantiate_classes(cfg)['cal']), Calendar)
+
+
+    def test_linking_deep_targets(self):
+        class D:
+            pass
+
+        class A:
+            def __init__(self, d: D) -> None:
+                self.d = d
+
+        class BSuper:
+            pass
+
+        class BSub(BSuper):
+            def __init__(self, a: A) -> None:
+                self.a = a
+
+        class C:
+            def fn(self) -> D:
+                return D()
+
+        config = {
+            "b": {
+                "class_path": "jsonargparse_tests.BSub",
+                "init_args": {
+                    "a": {
+                        "class_path": "jsonargparse_tests.A",
+                    },
+                },
+            },
+            "c": {},
+        }
+        config_path = os.path.join(self.tmpdir, 'config.yaml')
+        with open(config_path, 'w') as f:
+            yaml.safe_dump(config, f)
+
+        import jsonargparse_tests
+        setattr(jsonargparse_tests, 'D', D)
+        setattr(jsonargparse_tests, 'A', A)
+        setattr(jsonargparse_tests, 'BSuper', BSuper)
+        setattr(jsonargparse_tests, 'BSub', BSub)
+        setattr(jsonargparse_tests, 'C', C)
+
+        parser = ArgumentParser()
+        parser.add_argument("--config", action=ActionConfigFile)
+        parser.add_subclass_arguments(BSuper, nested_key="b", required=True)
+        parser.add_class_arguments(C, nested_key="c")
+        parser.link_arguments("c", "b.init_args.a.init_args.d", compute_fn=C.fn, apply_on="instantiate")
+
+        config = parser.parse_args(["--config", config_path])
+        config_init = parser.instantiate_classes(config)
+        self.assertIsInstance(config_init["b"].a.d, D)
+
+
+    def test_mapping_class_typehint(self):
+        class A:
+            pass
+
+        class B:
+            def __init__(
+                self,
+                class_map: Mapping[str, A],
+                int_list: List[int],
+            ):
+                self.class_map = class_map
+                self.int_list = int_list
+
+        import jsonargparse_tests
+        setattr(jsonargparse_tests, 'A', A)
+        setattr(jsonargparse_tests, 'B', B)
+
+        parser = ArgumentParser(error_handler=None)
+        parser.add_class_arguments(B, 'b')
+
+        config = {
+            'b': {
+                'class_map': {
+                    'one': {'class_path': 'jsonargparse_tests.A'},
+                },
+                'int_list': [1],
+            },
+        }
+
+        cfg = parser.parse_object(config)
+        self.assertEqual(cfg.b.class_map, {'one': Namespace(class_path='jsonargparse_tests.A')})
+        self.assertEqual(cfg.b.int_list, [1])
+
+        cfg_init = parser.instantiate_classes(cfg)
+        self.assertIsInstance(cfg_init.b, B)
+        self.assertIsInstance(cfg_init.b.class_map, dict)
+        self.assertIsInstance(cfg_init.b.class_map['one'], A)
+
+        config['b']['int_list'] = config['b']['class_map']
+        self.assertRaises(ParserError, lambda: parser.parse_object(config))
+
+
+    def test_linking_deep_targets_mapping(self):
+        class D:
+            pass
+
+        class A:
+            def __init__(self, d: D) -> None:
+                self.d = d
+
+        class BSuper:
+            pass
+
+        class BSub(BSuper):
+            def __init__(self, a_map: Mapping[str, A]) -> None:
+                self.a_map = a_map
+
+        class C:
+            def fn(self) -> D:
+                return D()
+
+        config = {
+            "b": {
+                "class_path": "jsonargparse_tests.BSub",
+                "init_args": {
+                    "a_map": {
+                        "name": {
+                            "class_path": "jsonargparse_tests.A",
+                        },
+                    },
+                },
+            },
+            "c": {},
+        }
+        config_path = os.path.join(self.tmpdir, 'config.yaml')
+        with open(config_path, 'w') as f:
+            yaml.safe_dump(config, f)
+
+        import jsonargparse_tests
+        setattr(jsonargparse_tests, 'D', D)
+        setattr(jsonargparse_tests, 'A', A)
+        setattr(jsonargparse_tests, 'BSuper', BSuper)
+        setattr(jsonargparse_tests, 'BSub', BSub)
+        setattr(jsonargparse_tests, 'C', C)
+
+        parser = ArgumentParser()
+        parser.add_argument("--config", action=ActionConfigFile)
+        parser.add_subclass_arguments(BSuper, nested_key="b", required=True)
+        parser.add_class_arguments(C, nested_key="c")
+        parser.link_arguments("c", "b.init_args.a_map.name.init_args.d", compute_fn=C.fn, apply_on="instantiate")
+
+        config = parser.parse_args(["--config", config_path])
+        config_init = parser.instantiate_classes(config)
+        self.assertIsInstance(config_init["b"].a_map["name"].d, D)
+
+        config_init = parser.instantiate_classes(config)
+        self.assertIsInstance(config_init["b"].a_map["name"].d, D)
 
 
 class OtherTests(unittest.TestCase):
